import boatwright
from MACD import MACD
from datetime import datetime, timedelta
import matplotlib.pyplot as plt
import numpy as np

broker = boatwright.Brokers.BacktestBroker(taker_fee=0, maker_fee=0, slippage=0, quote_symbol="USD")
strategy = MACD(fast_period=7, slow_period=14, symbol="BTC", broker=broker)
database = boatwright.Data.CSVdatabase(source="COINBASE")

backtests = []
end = datetime(year=2023, month=11, day=10, hour=1, minute=0)
days = np.arange(1,101,2)

for d in days:
    start = end - timedelta(days=int(d))
    data = database.load(symbol=strategy.symbol, start=start, end=end, prerequisite_data_length=strategy.calc_prerequisite_data_length(), granularity=1, granularity_unit="MINUTE")
    backtest = boatwright.Backtest(strategy=strategy, data=data)
    backtests.append(backtest)

executor = boatwright.BacktestExecutors.SequentialExecutor()
# executor = boatwright.BacktestExecutors.ParallelExecutor(n_procs=4)
backtests = executor.run(backtests, verbose=True)

n_bars = np.array([len(b.data) for b in backtests])
runtimes = np.array([b.runtime for b in backtests])

fig, ax = plt.subplots()
ax.set_title("backtest runtime vs duration")
ax.set_xlabel("number of bars")
ax.set_ylabel("backtest run time [s]")
ax.plot(n_bars, runtimes)
ax.legend(loc="best")

plt.show()
